package org.greenstone.gsdl3.gs3build.doctypes;

import org.w3c.dom.*;

import java.net.URL;

import java.util.List;
import java.util.ArrayList;

import org.greenstone.gsdl3.gs3build.metadata.METSStructureSet;
import org.greenstone.gsdl3.gs3build.metadata.AbstractStructure;
import org.greenstone.gsdl3.gs3build.metadata.METSStructure;
import org.greenstone.gsdl3.gs3build.metadata.METSDivision;
import org.greenstone.gsdl3.gs3build.metadata.METSDescriptive;
import org.greenstone.gsdl3.gs3build.metadata.METSFileSet;
import org.greenstone.gsdl3.gs3build.metadata.METSFileGroup;
import org.greenstone.gsdl3.gs3build.metadata.METSFile;
import org.greenstone.gsdl3.gs3build.metadata.METSFilePos;
import org.greenstone.gsdl3.gs3build.metadata.METSFileID;
import org.greenstone.gsdl3.gs3build.metadata.MetadataLabel;
import org.greenstone.gsdl3.gs3build.doctypes.AbstractDocument;
import org.greenstone.gsdl3.gs3build.util.*;

public class HTMLDocumentTools
{
  Document          document;
  URL               documentUrl;
  int               sectionLevel;
  StringBuffer      sectionLabel;
  List              sectionSeq;
  HTMLSection       rootSection;
  HTMLSection       currentSection;
  int               sectionSeqNo;
  METSStructure     metsStructure;
  AbstractStructure currentMets;
  AbstractDocument  metsDocument;

  /**
   *
   */
  class HTMLSection
  { String startLocation;
    String endLocation;
    String label;
    List   children;
    HTMLSection parent;
    int         seqNo;

    public HTMLSection(String label, int seqNo)
    { this.label = label;
      this.seqNo = seqNo;
      this.parent = null;
      this.children = new ArrayList();
    }
    
    public void setStartLocation(String location)
    { this.startLocation = location;
    }
    
    public void setEndLocation(String location)
    { this.endLocation = location;
    }

    public void addChild(HTMLSection child)
    { this.children.add(child);
      child.parent = this;
    }
    
    public HTMLSection getParent()
    { return this.parent;
    }

    public String getLabel()
    { return this.label;
    }

    public String getXPointer()
    { StringBuffer xPointerBuffer = new StringBuffer();
    
      xPointerBuffer.append("xpointer(");
      xPointerBuffer.append(this.startLocation);
      xPointerBuffer.append("/range-to(");
      xPointerBuffer.append(this.endLocation);
      xPointerBuffer.append("))");
      
      return xPointerBuffer.toString();
    }
  }

  public HTMLDocumentTools(Document document)
  { this.document = document;
    this.sectionLevel = 0;
    this.sectionSeq   = new ArrayList();
    this.sectionSeq.add(new Integer(0));
    this.sectionLabel = new StringBuffer();
    this.rootSection = new HTMLSection("", 0);
    this.currentSection = this.rootSection;
    this.metsStructure  = new METSStructure(METSStructureSet.GSDL3_SECTION_STRUCTURE, 
					    "Section", "Section");
    this.currentMets    = this.metsStructure;
    this.metsDocument   = null;
  }

  public void setMetsDocument(AbstractDocument document)
  { this.metsDocument = document;
  }

  public void setUrl(URL url)
  { this.documentUrl = url;
  }

  public Object findSections()
  { this.findSections(document.getDocumentElement(), "/1");
    return this.rootSection;
  }

  public void findSections(Node node, String location)
  { // if at an element, then we aren't going to find section information,
    // but we do need to recurse into the children to find the section markers
    // there
    if (node.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) {
      NodeList children = node.getChildNodes();
      StringBuffer locationBuffer = new StringBuffer();
      locationBuffer.append(location);
      locationBuffer.append("/");

      int elementNo = 1;
      int commentNo = 1;
      for (int c = 0; c < children.getLength(); c++)
      {
	locationBuffer.setLength(location.length()+1);
	if (children.item(c).getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) {
	  locationBuffer.append(Integer.toString(elementNo));
	  elementNo ++;
	}
	else if (children.item(c).getNodeType() == org.w3c.dom.Node.COMMENT_NODE) {
	  locationBuffer.append("comment()[");
	  locationBuffer.append(Integer.toString(commentNo));
	  locationBuffer.append("]");
	  commentNo ++;
	}
	this.findSections(children.item(c), locationBuffer.toString());
      }
    }
    // if at a comment node, then one needs to dissect it to discover if
    // any section markings are included...
    else if (node.getNodeType() == org.w3c.dom.Node.COMMENT_NODE) {
      String commentText = node.getNodeValue();
      
      // parse the node now...checking first for any 'section' items in
      // it - this is a crude check...
      if (commentText.indexOf("<Section>") >= 0 ||
	  commentText.indexOf("</Section>") >= 0) {
	// parse the section roughly using the primitive HTMLDoc form...      
	HTMLDoc innerDoc     = HTMLDoc.fromString("dummy", commentText);
	HTMLBlock innerHtml  = innerDoc.getCodedContent();
	boolean   inMetadata = false;
	StringBuffer metaValue = new StringBuffer();
	String       metaLabel = null;

	// Now just go through the content in sequence...
	for (int e = 0; e < innerHtml.size(); e ++) {
	  Object element = innerHtml.elementAt(e);

	  if (element instanceof HTMLTag) {
	    HTMLTag tag = (HTMLTag) element;

	    if (tag.tagName().equals("section")) {
	      // get the counters sorted out for the sections...
	      this.sectionSeq.set(this.sectionLevel, new Integer(((Integer) this.sectionSeq.get(this.sectionLevel)).intValue() + 1));
	      this.sectionLevel ++;
	      if (sectionSeq.size() == this.sectionLevel) {
		this.sectionSeq.add(new Integer(0));
	      }
	      else {
		this.sectionSeq.set(this.sectionLevel, new Integer(0));
	      }

	      // create the label for the new section...
	      this.sectionLabel.setLength(0);
	      for (int s = 0; s < sectionLevel; s ++) {
		if (s > 0) {
		  this.sectionLabel.append(".");
		}
		this.sectionLabel.append(this.sectionSeq.get(s).toString());
	      }

	      // note the new section in a new data structure...
	      HTMLSection newSection = new HTMLSection(this.sectionLabel.toString(), this.sectionSeqNo);

	      newSection.setStartLocation(location);
	      this.currentSection.addChild(newSection);
	      this.currentSection = newSection;

	      // do the same for mets...
	      METSDivision newDivision = new METSDivision("DS" + Integer.toString(this.sectionSeqNo), Integer.toString(this.sectionSeqNo),
							  this.sectionLabel.toString(), this.sectionLabel.toString(), "Section");
	      this.currentMets.addDivision(newDivision);
	      this.currentMets = newDivision;

	      this.sectionSeqNo ++;
	    }
	    else if (tag.tagName().equals("/section")) {
	      // cope with poorly formed metadata...
	      if (inMetadata) {
		this.postMetadata(metaLabel, metaValue.toString());
	      }

	      // note the end position of the section
	      this.currentSection.setEndLocation(location);

	      // now post that section to the file list...
	      this.postFilePtr(this.currentSection);

	      // backtrack up the section tree...
	      this.currentSection = this.currentSection.getParent();
	      this.currentMets = this.currentMets.getParent();
	      this.sectionLevel --;

	      // trim the section label
	      int cutTo = this.sectionLabel.toString().lastIndexOf('.');
	      if (cutTo >= 0) {
		this.sectionLabel.setLength(cutTo);
	      }
	      else {
		this.sectionLabel.setLength(0);
	      }

	      // we can't be in metadata either...
	      inMetadata = false;
	    }
	    else if (tag.tagName().equals("metadata")) {
	      if (inMetadata) {
		// post the metadata
		this.postMetadata(metaLabel, metaValue.toString());
	      }
	      metaValue.setLength(0);
	      metaLabel = tag.idValue("name");
	      inMetadata = (metaLabel != null);
	    }
	    else if (tag.tagName().equals("/metadata")) {
	      // post the metadata
	      this.postMetadata(metaLabel, metaValue.toString());
	      inMetadata = false;
	    }
	  }
	  else {
	    if (inMetadata) {
	      metaValue.append(element.toString());
	    }
	  }
	}
      }
    }
  }

  private String makeXPointer(HTMLSection section)
  { StringBuffer xPointerBuffer = new StringBuffer();

    if (this.documentUrl != null)
    { xPointerBuffer.append(this.documentUrl.toString());
      xPointerBuffer.append("#");
    }
    xPointerBuffer.append(section.getXPointer());

    return xPointerBuffer.toString();
  }

  private void postFilePtr(HTMLSection section)
  { String xPointer = this.makeXPointer(section);

    if (this.metsDocument == null) {
      return;
    }

    try {
      URL url = new URL(xPointer);

      METSFileSet fileSet = this.metsDocument.getDocumentFiles();

      METSFilePos filePos = new METSFilePos(url);
      METSFile    file = fileSet.createFile(filePos, "text/html");
      
      METSFileGroup fileGroup;
      String fileRef = ((METSDivision) this.currentMets).getDefaultFileReference();

      if (fileRef == null) {
	fileGroup = fileSet.createGroup();
	fileSet.addGroup(fileGroup);
	((METSDivision) this.currentMets).addFileReference(fileGroup.getId());
      }
      else {
	// simple call - the group should be at the 'root' level, immediately below
	// the FileSet itself...
	fileGroup = fileSet.getGroup(fileRef);
      }
      fileGroup.addFile(file);

      
    }
    catch (java.net.MalformedURLException ex) {
      System.err.println(ex + " " + xPointer);
    }
  }

  private void postMetadata(String label, String value)
  {
    if (this.metsDocument == null) {
      return;
    }

    String metadataRef = ((METSDivision) this.currentMets).getDefaultMetadataReference();
    METSDescriptive descriptive;
    if (metadataRef == null) {
      descriptive = this.metsDocument.getDocumentMetadata().createDescriptive(this.currentSection.getLabel());
      ((METSDivision) this.currentMets).addMetadataReference(descriptive.getID());
    }
    else {
      descriptive = this.metsDocument.getDocumentMetadata().getDescriptiveById(metadataRef);
    }
    descriptive.addMetadata(MetadataLabel.getDefaultNamespace(), label, value);
  }

  public METSStructure getStructure()
  { return this.metsStructure;
  }
}

